﻿; appinstall main file
; by felix caffier
; www.trisymphony.com
; 2015-02-25

; enable optional CLI output
OpenConsole()

; include applib
IncludeFile "../applib.pb"
; init applib
initInstalllib()

; AUTH CHECK - disable for debugging
If isRoot() Or isAppmin()
  ; continue program
Else
  MessageRequester(LANG_ERROR, "appinstall: "+LANG_ROOTAPPMIN)
  End
EndIf

; setting paths
If APPLIB_PARAMS(0) <> ""
  setInstallPathAndMode(APPLIB_PARAMS(0))
EndIf


; LOADING AND PREPROCESSING

; load basic form
XIncludeFile "scaleform.pbf"
; bring up "please wait" screen
XIncludeFile "waitform.pbf"
Openwinwait()

Procedure Preprocessing_Thread(*VAL)
  
  If Not scanTGZ()
    PostEvent(#InstallCheckFailed)
    ProcedureReturn
  EndIf

  createTempExtractFolder()
  ; read text files - DIM because TAR is faster if we all do it in 1 go
  ; READMEFILES is global, because PB procedures cannot return arrays
  extractFiles("README", "EULA", "LICENSE", "CREDITS") ; includes icon.png and info.xml
  
  PostEvent(#InstallCheckDone)
  
EndProcedure

T_PREP = CreateThread(@Preprocessing_Thread(), #PB_Any)

Procedure ContinueInstall()
  
  readme.s = READMEFILES(1)
  eula.s = READMEFILES(2)
  license.s = READMEFILES(3)
  credits.s = READMEFILES(4)
  
  ; close wait form
  CloseWindow(#winwait)
  
  ; resize window to desktop proportions
  ExamineDesktops()
  Openwinmain(0, 0, 900, DesktopHeight(0) - 300)
  ResizeGadgetswinmain()

  ; set correct "stepping" for install progress bar
  SetGadgetAttribute(#p_install, #PB_ProgressBar_Minimum, 0)
  SetGadgetAttribute(#p_install, #PB_ProgressBar_Maximum, APPLIB_TARFILESNUM)

  ; remove empty tabs - in reverse order
  If credits = ""
    RemoveGadgetItem(#infotabs, 3)
  Else
    SetGadgetText(#ed_credits, credits)
  EndIf
  If license = ""
    RemoveGadgetItem(#infotabs, 2)
  Else
    SetGadgetText(#ed_license, license)
  EndIf
  If eula = ""
    RemoveGadgetItem(#infotabs, 1)
  Else
    SetGadgetText(#ed_eula, eula)
  EndIf
  If readme = ""
    RemoveGadgetItem(#infotabs, 0)
  Else
    SetGadgetText(#ed_readme, readme)
  EndIf

  ; fill buttons
  If APPLIB_XMLEMAIL = ""
    DisableGadget(#b_mail, 1)
  EndIf
  If APPLIB_XMLWEBSITE = ""
    DisableGadget(#b_website, 1)
  EndIf

  ; fill info text
  infotext.s = APPLIB_XMLNAME
  If APPLIB_XMLAUTHOR <> ""
    infotext + LANG_BY + APPLIB_XMLAUTHOR
  EndIf
  If APPLIB_XMLVERSION <> ""
    infotext + LANG_VERSION + APPLIB_XMLVERSION
  EndIf
  If APPLIB_XMLBUILD <> ""
    infotext + LANG_BUILD + APPLIB_XMLBUILD
  EndIf

  If APPLIB_XMLDATE <> ""
    infotext + " (" + APPLIB_XMLDATE + ")"
  EndIf

  If APPLIB_XMLDESCRIPTION <> ""
    infotext + Chr(13) + Chr(13) + APPLIB_XMLDESCRIPTION
  EndIf

  SetGadgetText(#l_info, infotext)

  ; arch check - depends on launch-arch.sh
  If Not isArchSupported()
    DisableGadget(#b_install, 1)  
    MessageRequester(LANG_ERROR, LANG_ARCHUNSUPPORTED)
  EndIf
  
EndProcedure


  
  ; event loop
Repeat
  Event = WaitWindowEvent()
  
  Select Event
    Case #InstallSuccess
      PrintN(LANG_SUCCESS)
      SetGadgetText(#b_install, LANG_SUCCESS)
      res = MessageRequester(LANG_SUCCESS, LANG_SUCCESSTEXT)
;       PROBLEM: app is run as another user, so shortcut creation for "proper" user fails
;       res = MessageRequester(LANG_SUCCESS, LANG_SUCCESSTEXTSHORTCUT, #PB_MessageRequester_YesNo)
;       If res = #PB_MessageRequester_Yes
;         RunProgram("appdesktopshortcut", "/app/" + APPLIB_INSTALLPACKNAME, "/tmp", #PB_Program_Wait)
;       EndIf
    Case #InstallError
      PrintN(LANG_ERROR)
      SetGadgetText(#b_install, LANG_ERROR)
      MessageRequester(LANG_ERROR, LANG_INSTALLPERMISSIONDENIED)
    Case #InstallCheckFailed
      EndProgram(TAR)
      EndProgram(UNPACK)
      KillThread(T_PREP)
      MessageRequester(LANG_ERROR, LANG_ERRORTAR)
      End
    Case #InstallCheckDone
      ContinueInstall()
    Case #InstallAbort
      EndProgram(TAR)
      EndProgram(UNPACK)
      KillThread(T_PREP)
      MessageRequester(LANG_BWINWAIT, LANG_INSTALLFAIL)
      End
  EndSelect
  
  Select EventWindow()
    Case #winmain
      winmain_Events(Event)
    Case #winwait
      winwait_Events(Event)
  EndSelect
  
Until Event = #PB_Event_CloseWindow


; hack for "X" button in #winwait
EndProgram(TAR)
EndProgram(UNPACK)
KillThread(T_PREP)

End
; IDE Options = PureBasic 5.24 LTS (Linux - x64)
; CursorPosition = 142
; FirstLine = 125
; Folding = -
; EnableUnicode
; EnableThread
; EnableXP
; Executable = ../BUILD-x86_64/appinstall