﻿; procedure lib for the appsuite
; by felix caffier
; www.trisymphony.com
; 2015-01-24

UsePNGImageDecoder()

;--
;-- USER AND GROUPS FOR /APP
;--
Global APPLIB_APPMIN.s = "appmin"   ; admin for apps and owner of everything in /app
Global APPLIB_APPGROUP.s = "apps"   ; user group for everything in /app

;--
;-- ENUMERATIONS
;--
Enumeration infoXML
  #hxml
EndEnumeration

;--
;-- APP GLOBALS
;--
Global      APPLIB_APPMODE.s = ""   ; "apx" (GUI) or "cap" (Console) mode - app starter switch
Global      APPLIB_CLEANPATH.s = "" ; absolute path of app bundle
Global Dim  APPLIB_PARAMS.s (1)     ; program parameter storage - needs to be ReDim'd dynamically!
Global      APPLIb_ISOLang.s = "en" ; deafult language english
Global      APPLIB_ARCH.s = ""      ; APPLIB_ARCHitecture defined by uname -m, like x86_64, sparc, mipsel, etc.

;--
;-- INFO/INSTALLER APP GLOBALS
;--
Global          APPLIB_INFOMODE.s = "apx"      ; "apx" (GUI) or "cap" (Console) mode - display mode for the info app
Global NewList  APPLIB_TARFILES.s()            ; contains file list of installer package appname.apx.tar.gz
Global          APPLIB_TARFILESNUM = 0         ; # of files contained in installation tar.gz
Global          APPLIB_INSTALLSOURCE.s = ""    ; source tar.gz
Global          APPLIB_INSTALLTEMP.s  = ""     ; /tmp path for unpacked info files
Global          APPLIB_INSTALLPACK.s = ""      ; tar.gz name without the path
Global          APPLIB_INSTALLPACKNAME.s = ""  ; tar.gz name without the path and tar.gz (so only .apx and .cap)
Global Dim      READMEFILES.s (4)              ; holds readme, credits, license and eula texts 
Global          UNPACK                         ; thread handle for untar process
Global          TAR                            ; thread handle for untar count process

Enumeration #PB_Event_FirstCustomValue
  #InstallSuccess
  #InstallError
  #InstallCheck
  #InstallCheckDone
  #InstallCheckFailed
  #InstallAbort
EndEnumeration


;--
;-- XML GLOBALS
;--
Global APPLIB_XMLAUTHOR.s = ""        ; program author - person or company
Global APPLIB_XMLEMAIL.s = ""         ; program support mail address
Global APPLIB_XMLWEBSITE.s = ""       ; program website url
Global APPLIB_XMLNAME.s = ""          ; program name
Global APPLIB_XMLDESCRIPTION.s = ""   ; program description
Global APPLIB_XMLBUILD.s = ""         ; program build number
Global APPLIB_XMLVERSION.s = ""       ; program version
Global APPLIB_XMLDATE.s = ""          ; program date
Global APPLIB_XMLCATEGORY.s = ""      ; categories for desktop environment

;--
;-- XML MAPS
;--
Global NewMap APPLIB_DescLang.s()   ; assocArray of languagecodes -> program description
Global NewMap APPLIB_NameLang.s()   ; assocArray of languagecodes -> program name
Global NewMap APPLIB_InfoLang.s()   ; assocArray of languagecodes -> program info

;--
;-- AUTH
;--
Procedure isRoot()
  If getuid_() = 0
    ProcedureReturn 1 ; true - is root
  Else
    ProcedureReturn 0 ; false - some other user
  EndIf
EndProcedure

Procedure isAppmin()
  idAPPLIB_APPMIN = RunProgram("id", "-u "+APPLIB_APPMIN, "/tmp", #PB_Program_Open|#PB_Program_Read)
  If ReadProgramString(idAPPLIB_APPMIN) = Str(getuid_())
    ProcedureReturn 1 ; true - user id matches appmin id
  Else
    ProcedureReturn 0 ; false - either id mismatch or error message
  EndIf
EndProcedure

;--
;-- HELPER FUNCTIONS
;--
Procedure EndProgram(HANDLE)
  If IsProgram(HANDLE)
    KillProgram(HANDLE)
    CloseProgram(HANDLE)
  EndIf
EndProcedure

Procedure.s escapePath(path.s) ; quote paths in case there are spaces in them
  ProcedureReturn #DQUOTE$ + path + #DQUOTE$
EndProcedure

Procedure.s getLang () ; get 2 letter language code based on iso 639
  lang.s = GetEnvironmentVariable("LANG")
  If lang <> ""
    APPLIB_ISOLang = Mid(lang, 0, 2) ; first 2 letters constitute iso 639 language codes
  Else
    APPLIB_ISOLang = "en"
  EndIf
EndProcedure

Procedure getArch() ; get current architecture string
  APPLIB_ARCHprog = RunProgram("uname", "-m", GetCurrentDirectory(), #PB_Program_Open | #PB_Program_Read)
  APPLIB_ARCH = Trim(ReadProgramString(APPLIB_ARCHprog))
EndProcedure

Procedure setDefaultAppIcon() ; set program-wide icon from png file
  IconPath.s = APPLIB_CLEANPATH + "/icon.png"
  If FileSize(IconPath) > 0 And LoadImage(0, IconPath)
    gtk_window_set_default_icon_(ImageID(0))
  EndIf
EndProcedure

Procedure fillXMLMaps(*n) ; xml reader helper
  *node = ChildXMLNode(*n)
  While *node <> 0
  
    If XMLNodeType(*node) = #PB_XML_Normal
      nodename.s = GetXMLNodeName(*node)
    
      If nodename = "description"
        *child = ChildXMLNode(*node)
        While *child <> 0
          APPLIB_DescLang(GetXMLNodeName(*child)) = GetXMLNodeText(*child)
          *child = NextXMLNode(*child)
        Wend
      ElseIf nodename = "name"
        *child = ChildXMLNode(*node)
        While *child <> 0
          APPLIB_NameLang(GetXMLNodeName(*child)) = GetXMLNodeText(*child)
          *child = NextXMLNode(*child)
        Wend
      Else
        APPLIB_InfoLang(nodename) = GetXMLNodeText(*node)
      EndIf
    
    EndIf
    
    *node = NextXMLNode(*node)
  Wend
EndProcedure

;--
;-- BASE FUNCTIONS
;--
Procedure initApplib() ; first thing to call in a new applib program
  ; get language
  getLang ()
  IncludeFile "../applib-lang.pb"
  
  ; get computer APPLIB_ARCHitecture
  getArch()
  
  ; fill CLI parameter list
  num = CountProgramParameters()
  If num > 0
    ReDim APPLIB_PARAMS(num)
    For n = 0 To num-1
      APPLIB_PARAMS(n) = ProgramParameter()
    Next
  Else ; at least parameter 0 needs to be supplied, it is the app to start/analyze
    PrintN(LANG_USAGE1)
    PrintN(LANG_USAGE2)
    MessageRequester(LANG_ERROR, LANG_NOPARAMS)
    End
  EndIf
  
EndProcedure

Procedure initApplibLib() ; first thing to call in a new applib program, mute output
  ; get language
  getLang ()
  IncludeFile "../applib-lang.pb"
  
  ; get computer APPLIB_ARCHitecture
  getArch()
  
  ; fill CLI parameter list
  num = CountProgramParameters()
  If num > 0
    ReDim APPLIB_PARAMS(num)
    For n = 0 To num-1
      APPLIB_PARAMS(n) = ProgramParameter()
    Next
  Else
    End
  EndIf
  
EndProcedure

Procedure setAppPathAndMode(path.s) ; set APPLIB_CLEANPATH and APPLIB_APPMODE
  ; get rid of trailing slash, if any
  APPLIB_CLEANPATH = RTrim(Trim(path), "/")
  
  ; check for absolute path
  If Left(APPLIB_CLEANPATH, 1) <> "/"
    ; not an absolute path - assume the folder is already in /app/
    APPLIB_CLEANPATH = "/app/" + RTrim(Trim(path), "/")
  EndIf
  
  ; check if path is a directory
  If FileSize(path) = -2
    SetCurrentDirectory(path)
    
    ; get extension, presumably
    ext.s = Right(APPLIB_CLEANPATH, 3)
    If (ext = "apx" Or ext = "cap")
      APPLIB_APPMODE = ext
    Else
      MessageRequester(LANG_ERROR, LANG_MODE)
      End
    EndIf
    
  Else
    MessageRequester(LANG_ERROR, path+LANG_NODIR)
    End
  EndIf
EndProcedure

Procedure readAppXML() ; read info.xml and fill APPLIB_XML globals
  xmlname.s = APPLIB_CLEANPATH+"/info.xml"
  If LoadXML(#hxml, xmlname)
    If XMLStatus(#hxml) <> #PB_XML_Success
      ProcedureReturn 0
    EndIf
    
    *MainNode = MainXMLNode(#hxml)
    If *MainNode
      fillXMLMaps(*MainNode)
    EndIf
    
  EndIf
  
  ; fill xml globals from maps
  APPLIB_XMLAUTHOR = APPLIB_InfoLang("author")
  APPLIB_XMLEMAIL = APPLIB_InfoLang("email")
  APPLIB_XMLWEBSITE = APPLIB_InfoLang("website")
  APPLIB_XMLBUILD = APPLIB_InfoLang("build")
  APPLIB_XMLVERSION = APPLIB_InfoLang("version")
  APPLIB_XMLDATE = APPLIB_InfoLang("date")
  APPLIB_XMLCATEGORY = APPLIB_InfoLang("category")
  
  APPLIB_XMLNAME = APPLIB_NameLang(APPLIB_ISOLang)
  APPLIB_XMLDESCRIPTION = APPLIB_DescLang(APPLIB_ISOLang)
  ; if APPLIB_ISOLang returned an empty string, replace with default english
  If APPLIB_XMLNAME = ""
    APPLIB_XMLNAME = APPLIB_NameLang("en")
  EndIf
  If APPLIB_XMLDESCRIPTION = ""
    APPLIB_XMLDESCRIPTION = APPLIB_DescLang("en")
  EndIf
  
EndProcedure

Procedure.s getTextfile(uppercasename.s, NewLineCharacter.s = #CRLF$, BufferSize = 4096) ; read contents of text file to return string
  path.s = APPLIB_CLEANPATH+"/"+uppercasename+"_"+ UCase(APPLIB_ISOLang)
  If FileSize(path) < 1
    path = APPLIB_CLEANPATH+"/"+uppercasename
  EndIf
  
  out.s = ""
  If (path)
    fp = ReadFile(#PB_Any, path)
    If IsFile(fp)
      FileBuffersSize( fp, BufferSize )
      While Not Eof(fp)
        out + ReadString( fp ) + NewLineCharacter
      Wend
      CloseFile(fp)
    EndIf
    ProcedureReturn out
  EndIf
EndProcedure

;--
;-- INSTALLER FUNCTIONS
;--
Procedure isArchSupported() ; scan install .tar.gz file list for launch.sh or launch-arch.sh, return 0 or 1
  launchArch.s = APPLIB_INSTALLPACKNAME+"/launch-" + APPLIB_ARCH + ".sh"
  launchNoArch.s = APPLIB_INSTALLPACKNAME+"/launch.sh"
  
  ForEach APPLIB_TARFILES()
    If APPLIB_TARFILES() = launchArch Or APPLIB_TARFILES() = launchNoArch
      ; universal or APPLIB_ARCH-matching launch file found
      ProcedureReturn 1
      Break
    EndIf
  Next
  
  ; else - no suitable launch file found
  ProcedureReturn 0
EndProcedure

Procedure initInstalllib() ; first thing to call in a new applib installer program
  ; get language
  getLang ()
  IncludeFile "../applib-lang.pb"
  
  ; get computer APPLIB_ARCHitecture
  getArch ()
  
  ; fill CLI parameter list
  num = CountProgramParameters()
  If num > 0
    ReDim APPLIB_PARAMS(num)
    For n = 0 To num-1
      APPLIB_PARAMS(n) = ProgramParameter()
    Next
  Else ; no parameter supplied: ask for a package.
    install.s = OpenFileRequester(LANG_INSTALL_REQUESTER, GetHomeDirectory(), LANG_CAPP+"/"+LANG_XAPP+"|*.cap.tar.gz;*.apx.tar.gz|"+LANG_CAPP+"|*.cap.tar.gz|"+LANG_XAPP+"|*.apx.tar.gz", 0)
    If install
      APPLIB_INSTALLSOURCE = Trim(install)
      APPLIB_INSTALLPACK = GetFilePart(APPLIB_INSTALLSOURCE)
      APPLIB_INSTALLPACKNAME = Mid(APPLIB_INSTALLPACK, 1, Len(APPLIB_INSTALLPACK)-7) ; cut off ".tar.gz"
      APPLIB_INSTALLTEMP = "/tmp/appinstall/" + APPLIB_INSTALLPACKNAME
    Else
      PrintN(APPLIB_INSTALLPACKNAME + LANG_NODIR)
      End
    EndIf
  EndIf
  
EndProcedure

Procedure setInstallPathAndMode(path.s) ; sets APPLIB_INSTALL* paths
  ; get rid of trailing slash, if any
  APPLIB_INSTALLSOURCE = RTrim(Trim(path), "/")
  APPLIB_INSTALLPACK = GetFilePart(APPLIB_INSTALLSOURCE)
  APPLIB_INSTALLPACKNAME = Mid(APPLIB_INSTALLPACK, 1, Len(APPLIB_INSTALLPACK)-7) ; cut off ".tar.gz"
  APPLIB_INSTALLTEMP = "/tmp/appinstall/" + APPLIB_INSTALLPACKNAME
  
  ; check if tar.gz exists
  If Not FileSize(APPLIB_INSTALLSOURCE) > 0
    MessageRequester(LANG_ERROR, APPLIB_INSTALLPACKNAME+LANG_NODIR)
    End
  EndIf
EndProcedure

Procedure scanTGZ() ; fill APPLIB_TARFILES() with a list of file names included in the .tar.gz
  curfile.s = ""
  TAR = RunProgram ("tar", "-ztf " + escapePath(APPLIB_INSTALLSOURCE), "/tmp", #PB_Program_Open|#PB_Program_Read)
  While ProgramRunning(TAR)
    AddElement(APPLIB_TARFILES())
    curfile = ReadProgramString(TAR)
    APPLIB_TARFILES() = Trim(curfile)
;     PrintN(curfile) ; only for debugging
  Wend
  APPLIB_TARFILESNUM = ListSize(APPLIB_TARFILES())
  
  If APPLIB_TARFILESNUM > 0
    PrintN(LANG_FILESTOINSTALL+Str(APPLIB_TARFILESNUM))
    ProcedureReturn 1
  Else
    ProcedureReturn 0
  EndIf

EndProcedure

; only for single files - SLOW for multiple files! therefor deprecated
Procedure.s extractTextfile(uppercasename.s, NewLineCharacter.s = #CRLF$, BufferSize = 4096) ; extract text file from .tar.gz and read contents to return string
  ; seAPPLIB_ARCH for localized file in list
  loclang.s = APPLIB_INSTALLPACKNAME+"/"+uppercasename+"_"+ UCase(APPLIB_ISOLang)
  found = 0
  ForEach APPLIB_TARFILES()
    If APPLIB_TARFILES() = loclang
      found = 1
      Break
    EndIf
  Next
  ; else: no localized file found
  If found = 0
    loclang = APPLIB_INSTALLPACKNAME+"/"+uppercasename
    ForEach APPLIB_TARFILES()
      If APPLIB_TARFILES() = loclang
        found = 1
        Break
      EndIf
    Next
  EndIf
  ; still not found? escape!
  If found = 0
    ProcedureReturn ""
  EndIf
  
  ; otherwise, file found. now unpack
  untarfile = RunProgram("tar", "--extract " + escapePath("--file="+APPLIB_INSTALLSOURCE) + " -C /tmp/appinstall/ " + escapePath(loclang), "/tmp", #PB_Program_Wait)
  
  path.s = "/tmp/appinstall/"+loclang
  out.s = ""
  If (path)
    fp = ReadFile(#PB_Any, path)
    If IsFile(fp)
      FileBuffersSize( fp, BufferSize )
      While Not Eof(fp)
        out + ReadString( fp ) + NewLineCharacter
      Wend
      CloseFile(fp)
    EndIf
    ProcedureReturn out
  EndIf
  
EndProcedure

; multifile replacement for extractTextfile
Procedure extractFiles(uc1.s, uc2.s, uc3.s, uc4.s, NewLineCharacter.s = #CRLF$, BufferSize = 4096) ; extract text fileS
  ; README uc1
  ; EULA uc2
  ; LICENSE uc3
  ; CREDITS uc4
  Dim files.s (4)
  files(1) = uc1
  files(2) = uc2
  files(3) = uc3
  files(4) = uc4
  Dim tempfiles.s (ArraySize(files()))
  
  ; search for localized files in list
  For k=1 To ArraySize(files())
    tempfiles(k) = APPLIB_INSTALLPACKNAME+"/"+files(k)+"_"+ UCase(APPLIB_ISOLang)
    found = 0
    ForEach APPLIB_TARFILES()
      If APPLIB_TARFILES() = tempfiles(k)
        found = 1
        files(k) = files(k)+"_"+ UCase(APPLIB_ISOLang)
        Break
      EndIf
    Next
    ; not found? try unlocalized versions
    If found = 0
      tempfiles(k) = APPLIB_INSTALLPACKNAME+"/"+files(k)
      ForEach APPLIB_TARFILES()
        If APPLIB_TARFILES() = tempfiles(k)
          found = 1
          Break
        EndIf
      Next
    EndIf
    ; still not found? dummy value!
    If found = 0
      tempfiles(k) = ""
    EndIf
  Next
  
  ; cat all file names in one string
  unpackfiles.s = ""
  For k=1 To ArraySize(files())
    If tempfiles(k) = ""
      Continue
    Else
      unpackfiles + escapePath(tempfiles(k)) + " "
    EndIf
  Next
  
  ; now unpack ALL OF THEM
  UNPACK = RunProgram("tar", "--extract " + escapePath("--file="+APPLIB_INSTALLSOURCE) + " -C /tmp/appinstall/ " + unpackfiles + escapePath(APPLIB_INSTALLPACKNAME+"/info.xml") + " " + escapePath(APPLIB_INSTALLPACKNAME+"/icon.png"), "/tmp", #PB_Program_Wait)
  
  ; finally fill global Dim with texts
  For k=1 To ArraySize(files())
    path.s = "/tmp/appinstall/"+APPLIB_INSTALLPACKNAME+"/"+files(k)
    out.s = ""
    If (path)
      fp = ReadFile(#PB_Any, path)
      If IsFile(fp)
        FileBuffersSize( fp, BufferSize )
        While Not Eof(fp)
          out + ReadString( fp, #PB_UTF8 ) + NewLineCharacter
        Wend
        CloseFile(fp)
        READMEFILES(k) = out
      EndIf
    EndIf
  Next
  
  ; ------------------------------
  ; info.xml
  xmlname.s = "/tmp/appinstall/"+APPLIB_INSTALLPACKNAME+"/info.xml"
  If LoadXML(#hxml, xmlname)
    If XMLStatus(#hxml) <> #PB_XML_Success
      PostEvent(#InstallCheckFailed)
    EndIf
    
    *MainNode = MainXMLNode(#hxml)
    If *MainNode
      fillXMLMaps(*MainNode)
    EndIf
    
  EndIf
  
  ; fill xml globals from maps+
  APPLIB_XMLAUTHOR = APPLIB_InfoLang("author")
  APPLIB_XMLEMAIL = APPLIB_InfoLang("email")
  APPLIB_XMLWEBSITE = APPLIB_InfoLang("website")
  APPLIB_XMLBUILD = APPLIB_InfoLang("build")
  APPLIB_XMLVERSION = APPLIB_InfoLang("version")
  APPLIB_XMLDATE = APPLIB_InfoLang("date")
  APPLIB_XMLCATEGORY = APPLIB_InfoLang("category")
  
  APPLIB_XMLNAME = APPLIB_NameLang(APPLIB_ISOLang)
  APPLIB_XMLDESCRIPTION = APPLIB_DescLang(APPLIB_ISOLang)
  ; if APPLIB_ISOLang returned an empty string, replace with default english
  If APPLIB_XMLNAME = ""
    APPLIB_XMLNAME = APPLIB_NameLang("en")
  EndIf
  If APPLIB_XMLDESCRIPTION = ""
    APPLIB_XMLDESCRIPTION = APPLIB_DescLang("en")
  EndIf  
  
  ; ------------------------------
  ; default icon
  IconPath.s = "/tmp/appinstall/"+APPLIB_INSTALLPACKNAME+"/icon.png"
  If FileSize(IconPath) > 0 And LoadImage(0, IconPath)
    gtk_window_set_default_icon_(ImageID(0))
  EndIf
  
EndProcedure

Procedure createTempExtractFolder() ; create /tmp/appname.ext for text file/icon/xml extraction
  If FileSize(APPLIB_INSTALLTEMP) <> -2 ; folder does not exist - create it
    mk = RunProgram("mkdir", "-p " + #DQUOTE$ + APPLIB_INSTALLTEMP + #DQUOTE$, "/tmp")
  Else ; clean it
    ; DANGEROUS - you wouldn't want to go and delete important stuff... maybe remove that part?
    If Mid(APPLIB_INSTALLTEMP, 1,5) = "/tmp/"
      rm = RunProgram("rm", "-fr " + #DQUOTE$ + APPLIB_INSTALLTEMP + #DQUOTE$ + "/*", "/tmp")
    EndIf
  EndIf
EndProcedure

Procedure installFiles(*progbar) ; install to /app/progname.ext
  tarproc = RunProgram("tar", "-zxvf " + escapePath(APPLIB_INSTALLSOURCE) + " -C /app", "/tmp", #PB_Program_Open|#PB_Program_Read)
  
  curfile.s = ""
  state = 0
  tmp = 0
  
  While ProgramRunning(tarproc)
    curfile = ReadProgramString(tarproc)
    PrintN(curfile) ; optional - debug... but it's nice to have.
    
    If state >= APPLIB_TARFILESNUM
      Break
    EndIf
    
    If FindString(curfile, "tar: ") = 0 ; usually denotes an error message
      tmp = state + 1
      SetGadgetState(*progbar, tmp)
      state = tmp
      ; PrintN("STATE "+ Str(state)) ; DEBUG only
    Else
      PostEvent(#InstallError)
      ProcedureReturn
    EndIf
    
  Wend
  
  ; change permissions
  RunProgram("chown", "-R " + escapePath(APPLIB_APPMIN+":"+APPLIB_APPGROUP) +" "+ escapePath("/app/" + APPLIB_INSTALLPACKNAME), "/app/" + APPLIB_INSTALLPACKNAME, #PB_Program_Wait)
  RunProgram("chmod", "-R 751 " + escapePath("/app/" + APPLIB_INSTALLPACKNAME), "/app/" + APPLIB_INSTALLPACKNAME, #PB_Program_Wait)
  
  PostEvent(#InstallSuccess)
  ProcedureReturn
  
EndProcedure
; IDE Options = PureBasic 5.24 LTS (Linux - x64)
; CursorPosition = 199
; FirstLine = 97
; Folding = AMA9
; EnableUnicode
; EnableXP
; Executable = BUILD-x86_64/appdesktopshortcut