﻿; libmakegeneric
; create link chain for libs
; by felix caffier
; www.trisymphony.com
; 2015-03-26

; enable output
OpenConsole()

; include applib
IncludeFile "../applib.pb"

; init applib
initApplibLib()

Procedure makeLink(original.s, link.s)
  If FileSize(link) > 0
    ; do nothing i suppose
  Else
    ; file does not exist - create link
    RunProgram("ln", "-s " + escapePath(original) + " " + escapePath(link), original, #PB_Program_Wait)
  EndIf
EndProcedure

Procedure.s genLink1(original.s, name.s, prev.s = "")
  If name <> prev
    cur.s = ""
    If Not (Mid(name, Len(name)-2) = ".so")
      
      cur = ReplaceRegularExpression(4, name, "")
      If cur <> name
        PrintN(cur + " (-> "+original+")")
        makeLink (original, cur) ; MAKE LINK
        genLink1(original, cur, name) ; RECURSION
      EndIf
      
    EndIf
    
  EndIf
EndProcedure

Procedure.s genLink2(original.s, name.s, prev.s = "")
  If name <> prev
    cur.s = ""
    cur = ReplaceRegularExpression(5, name, ".so")
    If cur <> name
      PrintN(cur + " (-> "+original+")") 
      makeLink (original, cur) ; MAKE LINK  
      genLink2(original, cur, name) ; RECURSION
    Else
      final.s = ReplaceRegularExpression(6, name, ".so") ; last step, remove major version number
      PrintN(final + " (-> "+original+")")
      makeLink (original, final) ; MAKE LINK
    EndIf
    
  EndIf
EndProcedure

; AUTH CHECK - disable for debugging
If isRoot()
  ; continue program
Else
  PrintN(LANG_ERROR + ": ROOT permission required")
  End
EndIf

; get root folder with libs in it (presumably)
path.s = APPLIB_PARAMS(0)
path = RTrim(Trim(path), "/") + "/" ; make sure PATH has a trailing slash

; regex libtypes
CreateRegularExpression(1, path + "lib[a-zA-Z_0-9-]+[.]so[.0-9a-zA-Z]+") ; type 1: libSome_lib.so.6.5a
CreateRegularExpression(2, path + "lib[a-zA-Z_0-9]+-[-.0-9a-zA-Z]+[.]so$") ; type 2: libSome_lib-2.17a.so
CreateRegularExpression(3, path + "lib[a-zA-Z_0-9]+-[-.0-9a-zA-Z]+[.]so[.0-9a-zA-Z]+") ; type 3: mixed 1 + 2
; helper regex for capturing groups, since PB does not support capturing groups directly
CreateRegularExpression(4, "[.][0-9a-zA-Z]+$") ; captures last version bit of type 1, like .17a
CreateRegularExpression(5, "[.][0-9a-zA-Z]+[.]so$") ; captures last version bit of type 2, like .17a.so
CreateRegularExpression(6, "[-][0-9a-zA-Z]+[.]so$") ; captures last version bit of type 2, like -2.so

; storage for FIND output
NewList SoType1.s()
NewList SoType2.s()
NewList SoType3.s()
temp.s = ""

If FileSize(path) = -2 ; a folder
  find = RunProgram("find", escapePath(path) + " -type f", path, #PB_Program_Open| #PB_Program_Read)
  While ProgramRunning(find)
    temp = ReadProgramString(find)
      
    If MatchRegularExpression(1, temp)
      AddElement(SoType1())
      SoType1() = temp
        
    ElseIf MatchRegularExpression (2, temp)
      AddElement(SoType2())
      SoType2() = temp
      
    ElseIf MatchRegularExpression (3, temp)
      AddElement(SoType3())
      SoType3() = temp
      
    Else
      PrintN("DERP "+temp)
      
    EndIf

  Wend
  
Else ; not a folder: fail
  PrintN(path+": not a folder")
  End 1
EndIf


PrintN("TYPE 1---------------------")
ForEach SoType1()
  PrintN(SoType1())
  genLink1(SoType1(), SoType1())
Next

PrintN("TYPE 2---------------------")
ForEach SoType2()
  PrintN(SoType2())
  genLink2(SoType2(), SoType2())
Next

PrintN("TYPE 3---------------------")
ForEach SoType3()
  PrintN(SoType3())
  genLink1(SoType3(), SoType3())
Next

End
; IDE Options = PureBasic 5.24 LTS (Linux - x64)
; CursorPosition = 32
; FirstLine = 42
; Folding = -
; EnableUnicode
; EnableThread
; EnableXP
; Executable = ../BUILD-x86_64/libmakegeneric